@@ -70,28 +70,27 @@ static void
 get_volumes_cb (GObject *source_object, GAsyncResult *res, gpointer user_data)
 {
   GVfsAfpServer *server = G_VFS_AFP_SERVER (source_object);
-  GSimpleAsyncResult *simple = user_data;
-
+  GTask *task = G_TASK (user_data);
   GVfsBackendAfpBrowse *afp_backend;
   GPtrArray *volumes;
   GError *err = NULL;
-  
-  afp_backend = G_VFS_BACKEND_AFP_BROWSE (g_async_result_get_source_object (G_ASYNC_RESULT (simple)));
-  
+
+  afp_backend = G_VFS_BACKEND_AFP_BROWSE (g_task_get_source_object (task));
+
   volumes = g_vfs_afp_server_get_volumes_finish (server, res, &err);
   if (!volumes)
   {
-    g_simple_async_result_take_error (simple, err);
-    goto done;
+    g_task_return_error (task, err);
+    g_object_unref (task);
+    return;
   }
 
   if (afp_backend->volumes)
     g_ptr_array_unref (afp_backend->volumes);
   afp_backend->volumes = volumes;
 
-done:
-  g_simple_async_result_complete (simple);
-  g_object_unref (simple);
+  g_task_return_boolean (task, TRUE);
+  g_object_unref (task);
 }
 
 static void
@@ -100,13 +99,12 @@ update_cache (GVfsBackendAfpBrowse *afp_backend,
               GAsyncReadyCallback callback,
               gpointer user_data)
 {
-  GSimpleAsyncResult *simple;
-  
-  simple = g_simple_async_result_new (G_OBJECT (afp_backend), callback,
-                                      user_data, update_cache);
+  GTask *task;
 
-  g_vfs_afp_server_get_volumes (afp_backend->server, cancellable, get_volumes_cb,
-                                simple); 
+  task = g_task_new (afp_backend, cancellable, callback, user_data);
+  g_task_set_source_tag (task, update_cache);
+
+  g_vfs_afp_server_get_volumes (afp_backend->server, cancellable, get_volumes_cb, task);
 }
 
 static gboolean
@@ -114,18 +112,10 @@ update_cache_finish (GVfsBackendAfpBrowse *afp_backend,
                      GAsyncResult         *res,
                      GError              **error)
 {
-  GSimpleAsyncResult *simple;
-  
-  g_return_val_if_fail (g_simple_async_result_is_valid (res, G_OBJECT (afp_backend),
-                                                        update_cache),
-                        FALSE);
+  g_return_val_if_fail (g_task_is_valid (res, afp_backend), FALSE);
+  g_return_val_if_fail (g_async_result_is_tagged (res, update_cache), FALSE);
 
-  simple = (GSimpleAsyncResult *)res;
-
-  if (g_simple_async_result_propagate_error (simple, error))
-    return FALSE;
-
-  return TRUE;
+  return g_task_propagate_boolean (G_TASK (res), error);
 }
 
 static GVfsAfpVolumeData *
@@ -187,7 +177,7 @@ mount_mountable_cb (GObject      *source_object,
   if (!vol_data)
   {
     g_vfs_job_failed (G_VFS_JOB (job),  G_IO_ERROR, G_IO_ERROR_NOT_FOUND,
-                      _("File doesn't exist"));
+                      _("File doesn’t exist"));
     return;
   }
 
@@ -256,15 +246,10 @@ fill_info (GFileInfo *info, GVfsAfpVolumeData *vol_data, GVfsBackendAfpBrowse *a
   g_mount_spec_set (mount_spec, "user", afp_backend->logged_in_user);
 
   if (g_mount_tracker_has_mount_spec (afp_backend->mount_tracker, mount_spec))
-  {
     g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_MOUNTABLE_CAN_MOUNT, FALSE);
-    g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_MOUNTABLE_CAN_UNMOUNT, TRUE);
-  }
   else
-  {
     g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_MOUNTABLE_CAN_MOUNT, TRUE);
-    g_file_info_set_attribute_boolean(info, G_FILE_ATTRIBUTE_MOUNTABLE_CAN_UNMOUNT, FALSE);
-  }
+  g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_MOUNTABLE_CAN_UNMOUNT, FALSE);
   g_mount_spec_unref (mount_spec);
 
   uri = g_strdup_printf ("afp://%s/%s",
@@ -324,7 +309,7 @@ try_enumerate (GVfsBackend *backend,
   {
     g_vfs_job_failed (G_VFS_JOB (job),
                       G_IO_ERROR, G_IO_ERROR_NOT_FOUND,
-                      _("File doesn't exist"));
+                      _("File doesn’t exist"));
     return TRUE;
   }
 
@@ -356,7 +341,7 @@ query_info_cb (GObject      *source_object,
   if (!vol_data)
   {
     g_vfs_job_failed_literal (G_VFS_JOB (job),  G_IO_ERROR, G_IO_ERROR_NOT_FOUND,
-                              _("File doesn't exist"));
+                              _("File doesn’t exist"));
     return;
   }
 
@@ -518,7 +503,7 @@ g_vfs_backend_afp_browse_init (GVfsBackendAfpBrowse *object)
 {
   GVfsBackendAfpBrowse *afp_backend = G_VFS_BACKEND_AFP_BROWSE (object);
 
-  afp_backend->mount_tracker = g_mount_tracker_new (NULL);
+  afp_backend->mount_tracker = g_mount_tracker_new (NULL, FALSE);
 
   afp_backend->addr = NULL;
   afp_backend->user = NULL;
@@ -554,6 +539,7 @@ try_query_fs_info (GVfsBackend *backend,
                    GFileAttributeMatcher *matcher)
 {
   g_file_info_set_attribute_string (info, G_FILE_ATTRIBUTE_FILESYSTEM_TYPE, "afp");
+  g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_FILESYSTEM_REMOTE, TRUE);
   g_vfs_job_succeeded (G_VFS_JOB (job));
   return TRUE;
 }
