@@ -82,6 +82,10 @@ void              g_vfs_channel_send_reply         (GVfsChannel
 						    GVfsDaemonSocketProtocolReply *reply,
 						    const void                    *data,
 						    gsize                          data_len);
+void              g_vfs_channel_send_reply_take    (GVfsChannel                   *channel,
+                                                    GVfsDaemonSocketProtocolReply *reply,
+                                                    void                          *data,
+                                                    gsize                          data_len);
 guint32           g_vfs_channel_get_current_seq_nr (GVfsChannel                   *channel);
 GPid              g_vfs_channel_get_actual_consumer (GVfsChannel                  *channel);
 void              g_vfs_channel_force_close        (GVfsChannel                   *channel);
